from django.conf import settings
from django.db import IntegrityError
from django.contrib.auth.models import User


class SAMLServiceProviderBackend(object):

    def authenticate(self, saml_authentication=None):
        '''
        Username selection goes as follows:

        The default username for user@domain.tld is user. If the email for
        the user does not match, another user with the same email is assumed.

        To use a custom format for usernames, define them as a format string
        in LASTPASS_USERNAME_FORMAT. The result needs to be unique so use
        either "user", "domain" or "email" in your format string. Examples:

            # example-bob for bob@example.com
            LASTPASS_USERNAME_FORMAT = '{domain}-{user}'

            # alice-alice@example.com for alice@example.com (silly)
            LASTPASS_USERNAME_FORMAT = '{user}-{email}'

            # carol+filter@example.com
            LASTPASS_USERNAME_FORMAT = '{user}+filter@{domain}'

        If there are many users with the same email, it is assumed that the
        username is the email. The full email as username can be made the
        default by setting LASTPASS_FULL_EMAIL_USERNAME = True in Django
        settings.

        If there is no user matching the email by name or email, the attempt
        is given up and a new user gets created.

        A new user is created either with the name part of the email or if
        there is a user with that name, the email address is used.

        This makes sure that the LastPass users always get the best match for
        username.

        All users authenticated through LastPass will be made active.
        '''
        if not saml_authentication:  # Using another authentication method
            return None

        if saml_authentication.is_authenticated():
            superuser = getattr(settings, 'LASTPASS_CREATE_ADMIN', False)
            staff = getattr(settings, 'LASTPASS_CREATE_STAFF', False)

            email = saml_authentication.get_nameid()
            preferred_name, domain = email.split('@', 1)

            username_format = getattr(settings, 'LASTPASS_USERNAME_FORMAT', None)

            if username_format:
                preferred_name = username_format.format(email=email,
                                                        user=preferred_name,
                                                        domain=domain)
            elif getattr(settings, 'LASTPASS_USERNAME_FULL_EMAIL', False):
                preferred_name = email

            try:
                try:
                    user, created = User.objects.get_or_create(
                        email=email,
                        defaults={'username': preferred_name,
                                  'is_superuser': superuser,
                                  'is_staff': staff}
                    )
                except User.MultipleObjectsReturned:
                    user, created = User.objects.get_or_create(
                        username=preferred_name,
                        email=email,
                        defaults={
                            'username': email,
                            'is_superuser': superuser,
                            'is_staff': staff,
                            'email': email
                        }
                    )
            except IntegrityError:
                user, created = User.objects.get_or_create(
                    username=email,
                    defaults={
                        'is_superuser': superuser,
                        'is_staff': staff,
                        'email': email
                    }
                )

            if not user.is_active:
                user.is_active = True
                user.save()

            return user
        return None

    def get_user(self, user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None



from functools import wraps

from django.conf import settings
from django.core.urlresolvers import reverse

from saml_service_provider import views as samlviews
from .settings import LastPassServiceProviderSettings


def proxy_port(f):
    '''
    Patch saml_service_provider port setting

    saml_service_provider trusts too much in X-Forwarded-Port header
    and sets server port to local port if the header is missing.

    onelogin.saml2 explicitly sets the port eagerly.

    This results in some proxies being missed.

    This peeks common de facto reverse proxy headers and guesses
    the public port accordingly.
    '''
    @wraps(f)
    def wrapped(self, request, *f_args, **f_kwargs):
        if 'HTTP_X_FORWARDED_PORT' not in request.META:
            proto = request.META.get('HTTP_X_FORWARDED_PROTO', None)

            if 'HTTP_X_FORWARDED_FOR' in request.META:
                request.META['HTTP_X_FORWARDED_PORT'] = '80'

            if proto == 'https':
                request.META['HTTP_X_FORWARDED_PORT'] = '443'
            if proto == 'http':
                request.META['HTTP_X_FORWARDED_PORT'] = '80'

            if 'HTTP_X_FORWARDED_HOST' in request.META:
                try:
                    fwd_host = request.META['HTTP_X_FORWARDED_HOST']
                    port = int(fwd_host.rsplit(':', 1))
                    request.META['HTTP_X_FORWARDED_PORT'] = port
                except (ValueError, IndexError):
                    pass

        return f(self, request, *f_args, **f_kwargs)
    return wrapped


class SettingsMixin(object):
    def get_onelogin_settings(self):
        conf = dict()
        conf['onelogin_connector_id'] = settings.LASTPASS_CONNECTOR_ID
        if hasattr(settings, 'LASTPASS_CERTIFICATE'):
            conf['onelogin_x509_cert'] = settings.LASTPASS_CERTIFICATE
        elif hasattr(settings, 'LASTPASS_CERTIFICATE_FILE'):
            with open(settings.LASTPASS_CERTIFICATE_FILE, 'rb') as cert:
                conf['onelogin_x509_cert'] = cert.read()
        else:
            conf['onelogin_x509_fingerprint'] = settings.LASTPASS_FINGERPRINT

        hostname = getattr(settings, 'LASTPASS_HOSTNAME', 'http://localhost:8000')

        conf['sp_metadata_url'] = hostname + reverse('saml_metadata')
        conf['sp_login_url'] = hostname + reverse('saml_login_complete')
        conf['sp_logout_url'] = hostname + reverse('logout')
        conf['debug'] = settings.DEBUG
        conf['strict'] = not settings.DEBUG

        if hasattr(settings, 'SAML_SP_CERTIFICATE'):
            conf['sp_x509cert'] = settings.SAML_SP_CERTIFICATE
        else:
            with open(settings.SAML_SP_CERTIFICATE_FILE, 'rb') as cert:
                conf['sp_x509cert'] = cert.read()

        if hasattr(settings, 'SAML_SP_KEY'):
            conf['sp_private_key'] = settings.SAML_SP_KEY
        else:
            with open(settings.SAML_SP_KEY_FILE, 'rb') as key:
                conf['sp_private_key'] = key.read()

        if hasattr(settings, 'SAML_SP_CONTACT_INFO'):
            conf['contact_info'] = settings.SAML_SP_CONTACT_INFO

        if hasattr(settings, 'SAML_SP_ORGANIZATION_INFO'):
            conf['organization_info'] = settings.SAML_SP_ORGANIZATION_INFO

        return LastPassServiceProviderSettings(**conf).settings


class InitiateAuthenticationView(SettingsMixin,
                                 samlviews.InitiateAuthenticationView):
    @proxy_port
    def get(self, request, *args, **kwargs):
        return super(InitiateAuthenticationView, self).get(request, *args, **kwargs)


class CompleteAuthenticationView(SettingsMixin,
                                 samlviews.CompleteAuthenticationView):
    @proxy_port
    def post(self, request, *args, **kwargs):
        return super(CompleteAuthenticationView, self).post(request, *args, **kwargs)


class MetadataView(SettingsMixin,
                   samlviews.MetadataView):
    @proxy_port
    def get(self, request, *args, **kwargs):
        return super(MetadataView, self).get(request, *args, **kwargs)
